/****************************************************************************
*																			*
*							HPACK Multi-System Archiver						*
*							===========================						*
*																			*
*							  MD5 Message Digest Code						*
*							  MD5.C  Updated 02/10/91						*
*																			*
*   This program is protected by copyright and as such if you use or copy	*
*   this code for your own purposes directly or indirectly your soul will	*
*   become the property of the author with the right to dispose of it as	*
* 									he wishes.								*
*																			*
*			Copyright 1991  Peter C.Gutmann.  All rights reserved			*
*																			*
****************************************************************************/

#include <string.h>
#ifdef SYSV
  #include "../crypt/md5.h"
#else
  #include "crypt/md5.h"
#endif /* SYSV */

/****************************************************************************
*																			*
*	Implementation RSA Data Security, Inc. MD5 Message-Digest Algorithm		*
*	Created 2/17/90 RLR, revised 1/91 SRD,AJ,BSK,JT Reference C Version		*
*	  Copyright (C) 1990, RSA Data Security, Inc. All rights reserved		*
*																			*
****************************************************************************/

/* Define the following to use the assembly-language version of MDTransform().
   This is desirable since MDTransform() uses many rotates which cannot be
   easily represented in C and must be done inefficiently via shifts */

#ifdef __MSDOS__
  #define ASM_MD5
#endif /* __MSDOS__ */

#ifdef ASM_MD5
  void MD5Transform( LONG *buf, LONG *in );	/* Prototype for asm MDTransform() */
#endif /* ASM_MD5 */

/* To form the message digest for a message M, initialize a context buffer
   mdContext using MD5Init(); call MD5Update() on mdContext and M; and call
   MD5Final() on mdContext.  The message digest is now in
   mdContext->digest[ 0 ... 15 ] */

/****************************************************************************
*																			*
*							The MD5 Transformation							*
*																			*
****************************************************************************/

#ifndef ASM_MD5

/* F, G, H and I are basic MD5 functions */

#define F(X,Y,Z)	( ( X & Y ) | ( ~X & Z ) )
#define G(X,Y,Z)	( ( X & Z ) | ( Y & ~Z ) )
#define H(X,Y,Z)	( X ^ Y ^ Z )
#define I(X,Y,Z)	( Y ^ ( X | ~Z ) )

/* ROTATE_LEFT rotates x left n bits */

#define ROTATE_LEFT(x,n)	( ( x << n ) | ( x >> ( 32 - n ) ) )

/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.  Rotation is
   separate from addition to prevent recomputation */

#define FF(A,B,C,D,X,shiftAmt,magicConst) \
	{ \
	A += F( B, C, D ) + X + magicConst; \
	A = ROTATE_LEFT( A, shiftAmt ); \
	A += B; \
	}

#define GG(A,B,C,D,X,shiftAmt,magicConst) \
	{ \
	A += G( B, C, D ) + X + magicConst; \
	A = ROTATE_LEFT( A, shiftAmt ); \
	A += B; \
	}

#define HH(A,B,C,D,X,shiftAmt,magicConst) \
	{ \
	A += H( B, C, D ) + X + magicConst; \
	A = ROTATE_LEFT( A, shiftAmt ); \
	A += B; \
	}

#define II(A,B,C,D,X,shiftAmt,magicConst) \
	{ \
	A += I( B, C, D ) + X + magicConst; \
	A = ROTATE_LEFT( A, shiftAmt ); \
	A += B; \
	}

/* Round 1 shift amounts */

#define S11	7
#define S12	12
#define S13	17
#define S14	22

/* Round 2 shift amounts */

#define S21 5
#define S22 9
#define S23 14
#define S24 20

/* Round 3 shift amounts */

#define S31 4
#define S32 11
#define S33 16
#define S34 23

/* Round 4 shift amounts */

#define S41 6
#define S42 10
#define S43 15
#define S44 21

/* Basic MD5 step. Transforms buf based on in.  Note that if the Mysterious
   Constants are arranged backwards in little-endian order and decrypted with
   the DES they produce OCCULT MESSAGES! */

#if defined( IRIX ) || defined( ULTRIX )

static void MD5Transform( LONG *buf, LONG *in )
    {
    /* Do nothing - the MD5 code breaks the Irix and Ultrix compilers */
    }
    
#else

static void MD5Transform( LONG *buf, LONG *in )
	{
	LONG A = buf[ 0 ], B = buf[ 1 ], C = buf[ 2 ], D = buf[ 3 ];

	/* Round 1 */
	FF( A, B, C, D, in[  0 ], S11, 3614090360L );	/*  1 */
	FF( D, A, B, C, in[  1 ], S12, 3905402710L );	/*  2 */
	FF( C, D, A, B, in[  2 ], S13,  606105819L );	/*  3 */
	FF( B, C, D, A, in[  3 ], S14, 3250441966L );	/*  4 */
	FF( A, B, C, D, in[  4 ], S11, 4118548399L );	/*  5 */
	FF( D, A, B, C, in[  5 ], S12, 1200080426L );	/*  6 */
	FF( C, D, A, B, in[  6 ], S13, 2821735955L );	/*  7 */
	FF( B, C, D, A, in[  7 ], S14, 4249261313L );	/*  8 */
	FF( A, B, C, D, in[  8 ], S11, 1770035416L );	/*  9 */
	FF( D, A, B, C, in[  9 ], S12, 2336552879L );	/* 10 */
	FF( C, D, A, B, in[ 10 ], S13, 4294925233L );	/* 11 */
	FF( B, C, D, A, in[ 11 ], S14, 2304563134L );	/* 12 */
	FF( A, B, C, D, in[ 12 ], S11, 1804603682L );	/* 13 */
	FF( D, A, B, C, in[ 13 ], S12, 4254626195L );	/* 14 */
	FF( C, D, A, B, in[ 14 ], S13, 2792965006L );	/* 15 */
	FF( B, C, D, A, in[ 15 ], S14, 1236535329L );	/* 16 */

	/* Round 2 */
	GG( A, B, C, D, in[  1 ], S21, 4129170786L );	/* 17 */
	GG( D, A, B, C, in[  6 ], S22, 3225465664L );	/* 18 */
	GG( C, D, A, B, in[ 11 ], S23,  643717713L );	/* 19 */
	GG( B, C, D, A, in[  0 ], S24, 3921069994L );	/* 20 */
	GG( A, B, C, D, in[  5 ], S21, 3593408605L );	/* 21 */
	GG( D, A, B, C, in[ 10 ], S22,   38016083L );	/* 22 */
	GG( C, D, A, B, in[ 15 ], S23, 3634488961L );	/* 23 */
	GG( B, C, D, A, in[  4 ], S24, 3889429448L );	/* 24 */
	GG( A, B, C, D, in[  9 ], S21,  568446438L );	/* 25 */
	GG( D, A, B, C, in[ 14 ], S22, 3275163606L );	/* 26 */
	GG( C, D, A, B, in[  3 ], S23, 4107603335L );	/* 27 */
	GG( B, C, D, A, in[  8 ], S24, 1163531501L );	/* 28 */
	GG( A, B, C, D, in[ 13 ], S21, 2850285829L );	/* 29 */
	GG( D, A, B, C, in[  2 ], S22, 4243563512L );	/* 30 */
	GG( C, D, A, B, in[  7 ], S23, 1735328473L );	/* 31 */
	GG( B, C, D, A, in[ 12 ], S24, 2368359562L );	/* 32 */

	/* Round 3 */
	HH( A, B, C, D, in[  5 ], S31, 4294588738L );	/* 33 */
	HH( D, A, B, C, in[  8 ], S32, 2272392833L );	/* 34 */
	HH( C, D, A, B, in[ 11 ], S33, 1839030562L );	/* 35 */
	HH( B, C, D, A, in[ 14 ], S34, 4259657740L );	/* 36 */
	HH( A, B, C, D, in[  1 ], S31, 2763975236L );	/* 37 */
	HH( D, A, B, C, in[  4 ], S32, 1272893353L );	/* 38 */
	HH( C, D, A, B, in[  7 ], S33, 4139469664L );	/* 39 */
	HH( B, C, D, A, in[ 10 ], S34, 3200236656L );	/* 40 */
	HH( A, B, C, D, in[ 13 ], S31,  681279174L );	/* 41 */
	HH( D, A, B, C, in[  0 ], S32, 3936430074L );	/* 42 */
	HH( C, D, A, B, in[  3 ], S33, 3572445317L );	/* 43 */
	HH( B, C, D, A, in[  6 ], S34,   76029189L );	/* 44 */
	HH( A, B, C, D, in[  9 ], S31, 3654602809L );	/* 45 */
	HH( D, A, B, C, in[ 12 ], S32, 3873151461L );	/* 46 */
	HH( C, D, A, B, in[ 15 ], S33,  530742520L );	/* 47 */
	HH( B, C, D, A, in[  2 ], S34, 3299628645L );	/* 48 */

	/* Round 4 */
	II( A, B, C, D, in[  0 ], S41, 4096336452L );	/* 49 */
	II( D, A, B, C, in[  7 ], S42, 1126891415L );	/* 50 */
	II( C, D, A, B, in[ 14 ], S43, 2878612391L );	/* 51 */
	II( B, C, D, A, in[  5 ], S44, 4237533241L );	/* 52 */
	II( A, B, C, D, in[ 12 ], S41, 1700485571L );	/* 53 */
	II( D, A, B, C, in[  3 ], S42, 2399980690L );	/* 54 */
	II( C, D, A, B, in[ 10 ], S43, 4293915773L );	/* 55 */
	II( B, C, D, A, in[  1 ], S44, 2240044497L );	/* 56 */
	II( A, B, C, D, in[  8 ], S41, 1873313359L );	/* 57 */
	II( D, A, B, C, in[ 15 ], S42, 4264355552L );	/* 58 */
	II( C, D, A, B, in[  6 ], S43, 2734768916L );	/* 59 */
	II( B, C, D, A, in[ 13 ], S44, 1309151649L );	/* 60 */
	II( A, B, C, D, in[  4 ], S41, 4149444226L );	/* 61 */
	II( D, A, B, C, in[ 11 ], S42, 3174756917L );	/* 62 */
	II( C, D, A, B, in[  2 ], S43,  718787259L );	/* 63 */
	II( B, C, D, A, in[  9 ], S44, 3951481745L );	/* 64 */

	buf[ 0 ] += A;
	buf[ 1 ] += B;
	buf[ 2 ] += C;
	buf[ 3 ] += D;
	}
#endif /* IRIX || ULTRIX */

#endif /* ASM_MD5 */

/****************************************************************************
*																			*
*							MD5 Support Routines							*
*																			*
****************************************************************************/

/* The routine MD5Init initializes the message-digest context mdContext. All
   fields are set to zero */

void MD5Init( MD5_CTX *mdContext )
	{
	mdContext->i[ 0 ] = mdContext->i[ 1 ] = 0L;

	/* Load magic initialization constants */
	mdContext->buf[ 0 ] = 0x67452301L;
	mdContext->buf[ 1 ] = 0xEFCDAB89L;
	mdContext->buf[ 2 ] = 0x98BADCFEL;
	mdContext->buf[ 3 ] = 0x10325476L;
	}

/* The routine MD5Update updates the message-digest context to account for
   the presence of each of the characters inBuf[ 0 .. inLen-1 ] in the
   message whose digest is being computed.  This is an optimized version
   which assumes that the buffer is a multiple of MD5_BLOCKSIZE bytes long */

#ifdef __MSDOS__

void MD5Update( MD5_CTX *mdContext, BYTE *buffer, unsigned int noBytes )
	{
	int bufIndex = 0;

	/* Update number of bits */
	if( ( mdContext->i[ 0 ] + ( ( LONG ) noBytes << 3 ) ) < mdContext->i[ 0 ] )
		mdContext->i[ 1 ]++;	/* Carry from low to high bitCount */
	mdContext->i[ 0 ] += ( ( LONG ) noBytes << 3 );
	mdContext->i[ 1 ] += ( ( LONG ) noBytes >> 29 );

	/* Process data in MD5_BLOCKSIZE chunks */
	while( noBytes >= MD5_BLOCKSIZE )
		{
		MD5Transform( mdContext->buf, ( LONG * ) ( buffer + bufIndex ) );
		bufIndex += MD5_BLOCKSIZE;
		noBytes -= MD5_BLOCKSIZE;
		}

	/* Handle any remaining bytes of data.  This should only happen once
	   on the final lot of data */
	memcpy( mdContext->in, buffer + bufIndex, noBytes );
	}

/* The routine MD5Final terminates the message-digest computation and ends
   with the desired message digest in mdContext->digest[ 0 ... 15 ] */

void MD5Final( MD5_CTX *mdContext )
	{
	int mdi, padLen;
	LONG lowBitcount = mdContext->i[ 0 ], highBitcount = mdContext->i[ 1 ];

	/* Compute number of bytes mod 64 */
	mdi = ( int ) ( ( mdContext->i[ 0 ] >> 3 ) & 0x3F );
	padLen = ( mdi < 56 ) ? ( 56 - mdi ) : ( 120 - mdi );

	/* Update number of bits */
	if( ( mdContext->i[ 0 ] + ( ( LONG ) padLen << 3 ) ) < mdContext->i[ 0 ] )
		mdContext->i[ 1 ]++;	/* Carry from low to high bitCount */
	mdContext->i[ 0 ] += ( ( LONG ) padLen << 3 );
	mdContext->i[ 1 ] += ( ( LONG ) padLen >> 29 );

	/* Pad out to 56 mod 64 */
	if( padLen )
		{
		/* Set first char of padding to 0x80 */
		mdContext->in[ mdi ] = 0x80;

		if( mdi >= 56 )
			{
			/* Two lots of padding:  Pad first block to 64 bytes, then fill
			   next block with 56 bytes */
			memset( &mdContext->in[ mdi + 1 ], 0, 63 - mdi );
			MD5Transform( mdContext->buf, ( LONG * ) mdContext->in );
			memset( mdContext->in, 0, 56 );
			}
		else
			/* Pad block to 56 bytes */
			memset( &mdContext->in[ mdi + 1 ], 0, padLen - 1 );
		}
	if( mdi + padLen == 64 )
		MD5Transform( mdContext->buf, ( LONG * ) mdContext->in );

	/* Append length in bits and transform */
	( ( LONG * ) mdContext->in )[ 14 ] = lowBitcount;
	( ( LONG * ) mdContext->in )[ 15 ] = highBitcount;

	MD5Transform( mdContext->buf, ( LONG * ) mdContext->in );

	/* Store buffer in digest */
	memcpy( mdContext->digest, mdContext->buf, 16 );
	}

#else

void MD5Update( MD5_CTX *mdContext, BYTE *inBuf, unsigned int inLen )
	{
	int mdi;
	LONG in[ 16 ];
	unsigned int i, ii;

	/* Compute number of bytes mod 64 */
	mdi = ( int ) ( ( mdContext->i[ 0 ] >> 3 ) & 0x3F );

	/* Update number of bits */
	if( ( mdContext->i[ 0 ] + ( ( LONG ) inLen << 3 ) ) < mdContext->i[ 0 ] )
		mdContext->i[ 1 ]++;	/* Carry from low to high bitCount */
	mdContext->i[ 0 ] += ( ( LONG ) inLen << 3 );
	mdContext->i[ 1 ] += ( ( LONG ) inLen >> 29 );

	while( inLen-- )
		{
		/* Add new character to buffer, increment mdi */
		mdContext->in[ mdi++ ] = *inBuf++;

		/* Transform if necessary */
		if( mdi == 0x40 )
			{
			for( i = 0, ii = 0; i < 16; i++, ii += 4 )
				in[ i ] = ( ( ( LONG ) mdContext->in[ ii + 3 ] ) << 24 ) | \
						  ( ( ( LONG ) mdContext->in[ ii + 2 ] ) << 16 ) | \
						  ( ( ( LONG ) mdContext->in[ ii + 1 ] ) << 8 ) | \
						  ( ( LONG ) mdContext->in[ ii ] );
			MD5Transform( mdContext->buf, in );
			mdi = 0;
			}
		}
	}

/* The routine MD5Final terminates the message-digest computation and ends
   with the desired message digest in mdContext->digest[ 0 ... 15 ] */

void MD5Final( MD5_CTX *mdContext )
	{
	int mdi, padLen;
	BYTE padding[ 64 ];
	unsigned int i, ii;
	LONG in[ 16 ];

	/* Save number of bits */
	in[ 14 ] = mdContext->i[ 0 ];
	in[ 15 ] = mdContext->i[ 1 ];

	/* Compute number of bytes mod 64 */
	mdi = ( int ) ( ( mdContext->i[ 0 ] >> 3 ) & 0x3F );

	/* Pad out to 56 mod 64 */
	padLen = ( mdi < 56 ) ? ( 56 - mdi ) : ( 120 - mdi );
	padding[ 0 ] = 0x80;
	memset( padding + 1, 0, padLen - 1 );
	MD5Update( mdContext, padding, padLen );

	/* Append length in bits and transform */
	for( i = 0, ii = 0; i < 14; i++, ii += 4 )
		in[ i ] = ( ( ( LONG ) mdContext->in[ ii + 3 ] ) << 24 ) | \
				  ( ( ( LONG ) mdContext->in[ ii + 2 ] ) << 16 ) | \
				  ( ( ( LONG ) mdContext->in[ ii + 1 ] ) << 8 ) | \
				  ( ( LONG ) mdContext->in[ ii ] );
	MD5Transform( mdContext->buf, in );

	/* Store buffer in digest */
	for( i = 0, ii = 0; i < 4; i++, ii += 4 )
		{
		mdContext->digest[ ii ] = ( BYTE ) ( mdContext->buf[ i ] & 0xFF );
		mdContext->digest[ ii + 1 ] = ( BYTE ) ( ( mdContext->buf[ i ] >> 8 ) & 0xFF );
		mdContext->digest[ ii + 2 ] = ( BYTE ) ( ( mdContext->buf[ i ] >> 16 ) & 0xFF );
		mdContext->digest[ ii + 3 ] = ( BYTE ) ( ( mdContext->buf[ i ] >> 24 ) & 0xFF ); /**/
		}
	}
#endif /* !__MSDOS__ */
